;( function($) {
	'use strict';
	
	$.fn.lightbox = function(options) {
		
		options = $.extend({}, $.fn.lightbox.defaults, options);

		var text = getTranslations($.fn.lightbox.text),
			thumbs = $([]),
			thumb = $(),
			image = $(),
			media = $(),
			caption = $(),
			el,
			lightboxWidth,
			lightboxHeight,
			isImage = true,
			isVideo = false,
			isAudio = false,
			isOther = false,
			swiped = false,
			slideshow,
			fitTimeout,
			
			curr = -1,
			loadCounter = 0,
			loadTimer,
			direction = 0,

			ns = $(this).data('llb_ns'),
			extraSizes = options.hasOwnProperty('extraSizes')? options.extraSizes.split(/,\s*/) : false,

			cleanup = function(ns) {	
				// cleaning up the old event handlers
				$(window).add(document).off('.' + ns);
				// Removing the lightbox :: you can only have one lightbox per thumb set
				$('#' + ns).remove();
			},
			
			closeModal = function(id) {
				lightbox.find('.modal').trigger('destroy');
			},
			
			// Reacalculating image position and size upon window resize
			
			fitImage = function(initial) {
				
				var w, 
					h, 
					lbh,
					ratio, 
					img = image[0];
				
				clearTimeout(fitTimeout);
				
				if (!image || !image.length) {
					return;
				}
								
				if (!(w = image.data('oWidth'))) {
					w = (typeof img.naturalWidth === UNDEF)? (img.width || image.width()) : img.naturalWidth;
				}
				if (!(h = image.data('oHeight'))) {
					h = (typeof img.naturalHeight === UNDEF)? (img.height || image.height()) : img.naturalHeight;
				}
				
				if (w && h) {
				
					image.data({
						'oWidth': 	w,
						'oHeight': 	h
					});
					
					lbh = lightboxHeight - ((caption.length && options.fitAboveCaption)? caption.outerHeight() : 0);
					
					ratio = Math.min((lightboxWidth * options.fitRatio) / w, (lbh * options.fitRatio) / h);
	
					if (options.fitImages && !isOther) {
						if (ratio < 1 || options.scaleUp) {
							w = Math.round(w * ratio);
							h = Math.round(h * ratio);
						}
					}
					
					if (initial === true) {
						image.css({
							left: 		Math.round((lightboxWidth - w) / 2),
							top: 		Math.round((lbh - h) / 2),
							width: 		w,
							height: 	h,
							opacity: 	0
						}).show();
					} else {
						image.stop(true, false).animate({
							left: 		Math.round((lightboxWidth - w) / 2),
							top: 		Math.round((lbh - h) / 2),
							width: 		w,
							height: 	h
						}, 300, function() {
							$(this).translateX(0, 300)
						});
					}
					
					/*
					log('fit:'+lightbox.width() + ':' + lightbox.height());
					log('window:'+$(window).width() + ':' + $(window).height());
					*/
					
					$.cookie('fitImages', options.fitImages);
					
					lb_zoom.toggleClass('active', ratio >= options.fitRatio && options.fitImages || (ratio < 1 && !options.fitImages))
						.toggleClass('invisible', isOther || ratio >= 1 && !options.scaleUp);
				
				} else if (initial) {
					// Wait for width and height get value
					fitTimeout = setTimeout(fitImage, true, 100);
				}

			},
			
			// Removing the old image and caption
			
			removeOldContent = function() {
				
				var oldImage = lightbox.find('.lightbox-main'),
					oldCaption = lightbox.find('.lightbox-caption');
					
				if (oldImage.length) {
					
					// Removing the old image
					if (!swiped) {
						oldImage.translateXAndFade(-100 * direction, 0, options.speed, function() {
							oldImage.remove();	
						});
					}
					oldImage.off('.' + ns);
				}
				
				if (oldCaption.length) {
					
					// Removing the old caption
					oldCaption.find('.buttons a').trigger('removeTooltip');
					
					if (oldCaption.length > 1) {
						oldCaption.eq(-1).prevAll('.lightbox-caption').remove();
						oldCaption = lightbox.find('.lightbox-caption');
					}
					
					oldCaption.opacity(0, options.speed, function() {
						oldCaption.remove();	
					});
				}
			},
			
			// Reading image type
			
			getImageType = function(item) {
				isVideo = item[J.CATEGORY] === 'video';
				isAudio = item[J.CATEGORY] === 'audio';
				isOther = item[J.CATEGORY] === 'other';
				isImage = !isVideo && !isAudio && !isOther;
			},
			
			// Creating caption and buttons
			
			prepareCaption = function(item) {
				
				caption = $('<div>', {
						'class': 	'lightbox-caption' + (slideshow? ' slideshow' : '')
					}).hide().appendTo(lightbox);
				
				var buttons = $('<div>', {
						'class': 	'buttons'
					});

				if (extraSizes && item[J.CATEGORY] === 'image') {
					el = ($('<div>', {
							'class': 	'download icon-download'
						})).appendTo(buttons);
					
					var p = Album.getImagePath(item);
					
					for (var i = 0; i < extraSizes.length; i++) {
						el.append($('<a>', {
								text: 		extraSizes[i],
								href:  		p.replace('slides/', 'dl/' + extraSizes[i] + '/'),
								download: 	''
							}));
					}
				}
				
				if (options.share) {
					buttons.renderShares({
							sites: 			options.share,
							title: 			item[J.TITLE] || item[J.NAME],
							description: 	item[J.COMMENT],
							image: 			Album.getAbsoluteImagePath(item),
							href: 			Album.getAbsolutePath(item)
						});
				}
									
				if (options.printImage && item[J.CATEGORY] === 'image') {
					el = $('<a>', {
							'class': 	options.buttonClass + ' icon-printer',
							text: 		' ' + text.print
						}).appendTo(buttons);
					
					el.on('click', function(e) {
							printImage((item[J.ORIGINAL] && !options.rightClickProtect)? item[J.ORIGINAL][J.PATH] : item[J.IMAGE][J.PATH],
								item[J.TITLE] || '',
								item[J.THUMBCAPTION] || ''
							);
						});
				}
			
				if (item[J.ORIGINAL] && !options.rightClickProtect && 
					(item[J.CATEGORY] === 'image' || options.allowDownloadOthers)) {
					buttons.append($('<a>', {
							'class': 	options.buttonClass + ' icon-download',
							text: 		' ' + text.download,
							download: 	'',
							href: item[J.ORIGINAL][J.PATH]
						}));
				}
				
				if (options.hasOwnProperty('shop') && thumb.find('.add-cart').length) {
					el = $('<a>', {
							'class': 	options.buttonClass + ' icon-shopping-cart',
							text: 		' ' + text.addCart
						}).appendTo(buttons);
					
					el.on('click', function(e) {
							var btn = $(e.target);
								
							options.shop.root.trigger('addItems', item);
							
							if (!$.cookie('shoppingCartInfo')) {
								lightbox.modal($('<div>', {
									html: 	text.shoppingcartInfo
								}));
								
								$.cookie('shoppingCartInfo', true, 86400);
							}
							
							btn.addClass('disabled').text(text.added);
							setTimeout(function() {
								btn.removeClass('disabled').text(text.addCart);
							}, 1000);
						});
				}
				
				if (item[J.PHOTODATA]) {
					el = $('<a>', {
							'class': 	options.buttonClass + ' icon-camera',
							text: 		' ' + text.photoData
						}).appendTo(buttons);
					
					el.on('click', function() {
							lightbox.modal($('<div>', {
								'class':	'photodata',
								html: 		item[J.PHOTODATA]
							}), {
								'class':	'small',
								title: 		text.photoData
							});
							return false;
						});
				}
				
				if (item[J.LOCATION]) {
					el = $('<a>', {
						'class': 	options.buttonClass + ' icon-location',
						text: 		' ' + text.map
					}).appendTo(buttons);
					
					el.on('click', function() {
						var target = $('<div>', {
									'class': 'map-cont'
								});
						
						target.height($(window).height() * options.mapHeight);
						lightbox.modal(target, {
								'class': 'no-padding'
							});
						
						setTimeout(function() {
							target.addMap({
								location: 		item[J.LOCATION],
								type:			options.mapType,
								zoom:			options.mapZoom,
								fitBounds:		false,
								onTypeChanged:	function(type) {
													options.mapType = type;
												},
								onZoomChanged:	function(zoom) {
													options.mapZoom = zoom;
												}
							});
						}, 100);
						
						return false;
					});
				}
				
				if (item[J.MOSTPHOTOS]) {
					buttons.append($('<a>', {
							'class': 	options.buttonClass + ' icon-shopping-cart',
							text: 		' ' + text.buyImage,
							target: 	'_blank',
							href: 		'http://mostphotos.com/' + item[J.MOSTPHOTOS]
						}));
				}
				
				if (!buttons.is(':empty')) {
					caption.addClass('hasbuttons');
					caption.append(buttons);
				}
				
				if (item[J.IMAGECAPTION]) {
					caption.append($('<div>', {
							'class': 	'caption',
							html: 		item[J.IMAGECAPTION]
						}));
				}
				
				caption.toggle(!slideshow || caption.children('.caption').length > 0);
			},	
			
			// Preloading the next image (direction sensitive)
			
			preloadNext = function() {
				var nextItem;
				if (direction < 0 && curr > 0) {
					nextItem = thumbs.eq(curr - 1).data(J.OBJ);
				} else if (curr < thumbs.length - 2) {
					nextItem = thumbs.eq(curr + 1).data(J.OBJ);
				}
				if (nextItem) {
					if (nextItem[J.CATEGORY] === 'image') {
						var next = new Image();
						next.src = Album.getImagePath(nextItem);
					}
				}
			},
			
			// Image clicked :: toggle controls or audio/video controls
			
			imageClicked = function(e) {
				var touched = e.type !== 'mouseup';
							
				if (isVideo || isAudio) {
					
					var m = media[0];
					if (m.paused) {
						m.play();
						m.controls = false;
						/*
						hideCaption();
						if (touched) {
							hideControls();
						}
						*/
						hideControlsLater();
					} else {
						m.pause();
						m.controls = true;
						showCaption();
						//if (touched) {
							showControls();
						//}
					}
					
				} else if (isImage) {
					
					if (touched) {
						if (!caption.is(':visible') || caption.css('opacity') < 1) {
							showControls();
							showCaption();
						} else {
							hideControls();
							hideCaption();
						}
					} else {
						// image navigation
						if (((e.pageX || e.originalEvent.pageX) - image.position().left) > (image.width() / 2)) {
							nextImage();
						} else {
							previousImage();
						}
					}
				}
			},
			
			// The image is loaded
			
			imageReady = function() {
				
				loadCounter--;
				
				if (loadCounter > 0) {
					return;
				}
								
				if (DEBUG) {
					var d = new Date();
					console.log(((isVideo || isAudio)? 'Media' : 'Image') + ' [' + curr + '] loaded: ' + (d - loadTimer) + 'ms src="' + image[0].src + '"');
					if (typeof image[0] === UNDEF || typeof media[0] === UNDEF) {
						console.log('Premature ' + (typeof image[0] === UNDEF)? ('loadImage.done(' + image[0].src + ')') : ('loadMedia.done('+ media[0].src + ')'));
					}
				}
				
				image.css({
						opacity: 0
					}).hide();
				
				if (options.rightClickProtect) {
					media.on('contextmenu', function(e) {
							e.preventDefault()
							return false;
						});
				}
											
				if (isVideo || isAudio) {
					media.attr('autoplay', options.videoAuto);
					media[0].volume = options.volume;
					media[0].onvolumechange = function() {
							options.volume = this.volume;
						};
					hideControlsLater();
				}

				// Moving 100px away from center to prepare for moving back
				image.translateX(100 * direction);
				
				// Leaving time for CSS transform to settle down
				setTimeout(function() {
						
					fitImage(true);
					
					lb_activity.hide();
					
					// Moving back to center
					image.show().translateXAndFade(0, 1, options.speed);
					
					// Preparing the image after transition
					setTimeout(function() {
							
						// Swipe handler
						image.swipe({
							onSwipedLeft: 	function() {
												swiped = true;
												nextImage();
											},
											
							onSwipedRight: function() {
												swiped = true;
												previousImage();
											},
											
							onFinished: 	function() {
												$(this).trigger('removeSwipe');
												$(this).remove();
											},
											
							onClick:		imageClicked
						});

						if (options.onLoadEnd !== false) {
							options.onLoadEnd(thumb);
						}
						
						if (options.preloadNext) {
							preloadNext();
						}
						
					}, options.speed / 2);
					
				}, 50);
				
				// Continuing slideshow
				if (slideshow) {
					if (isAudio || isVideo) {
						suspendAuto();
						media[0].onended = function() {
							resumeAuto();
						};
					} else {
						clearTimeout(slideshow);
						lb_pause.find('.progress').show();
						slideshow = setTimeout(nextImage, options.slideshowDelay);
					}
				} else {
					stopAuto();
				}
			},
			
			// Main entry point to load a new image
			
			loadImage = function(n) {
				
				if (typeof n !== 'number') {
					n = thumbs.index(n);
				}
				
				if (n < 0 || n > thumbs.length) {
					// Out of bounds move
					if (n < 0 && options.onBeforeFirst !== false) {
						options.onBeforeFirst(lightbox);
					} else if (options.onAfterLast !== false) {
						options.onAfterLast(lightbox);
					}
					quitLightbox();
					return;
				}
				
				// Direction is calculated if the current thumb exists
				if (lightbox.is(':visible')) {
					// the previous image is still on screen
					direction = (curr > n)? -1 : ((curr < n)? 1 : 0);
					
					if (curr === n) {
						// the requested image is already on screen :: nothing to do
						return;
					}
				} else {
					// show lightbox
					$('body').css('overflow', 'hidden');
					direction = 0;
					lightbox.show();
				}
				
				closeModal();
				
				var item;
				
				curr = n;
				thumb = thumbs.eq(curr);
				item = thumb.data(J.OBJ);
				
				if (!item) {
					console.log('Fatal error: image (' + curr + ') is missing from the database! (Upload data1.json again!)');
					return;
				}
				
				lb_activity.show();
				lb_pause.find('.progress').hide();
				
				getImageType(item);
				
				if (options.onLoadStart !== false) {
					options.onLoadStart(thumb);
				}
				
				setTimeout(function() {
					
					// Delayed image loading
										
					removeOldContent(direction);
					swiped = false;

					loadCounter = 1;
					loadTimer = new Date();
					
					if (isVideo) {
						
						// Video
						
						media = image = $('<video>', {
								'class': 'lightbox-main video',
								preload: 'auto',
								controls: 'true',
								poster: Album.getImagePath(item)
							})
							.one('loadedmetadata', function(e) {
								imageReady();
							})
							.hide()
							.attr('src', Album.getSourcePath(item))
							.appendTo(lightbox);
						
					} else if (isImage) {
						
						// Image
						
						media = image = $('<img>', {
								'class': 'lightbox-main image'
							})
							.one('load.' + ns, function(e) {
								imageReady();
							})
							.hide()
							.attr('src', Album.getImagePath(item))
							.appendTo(lightbox);
						
						if (DEBUG) {
							console.log('Loading image [' + curr + '] src="' + Album.getImagePath(item) + '"');
						}
						
						if (image[0].complete && image[0].naturalWidth > 0) {
							// From cache
							image.off('load.' + ns);
							imageReady();
						}
						
					} else {
						
						// Audio or Other
						
						image = $('<div>', {
								'class': 'lightbox-main ' + item[J.CATEGORY]
							})
							.hide()
							.appendTo(lightbox);
						
						if (isAudio) {
							
							// Audio
							loadCounter = 2;
							$('<img>', {
									'class': 'poster'
								}).one('load', function(e) {
									imageReady();
								})
								.attr('src', Album.getImagePath(item))
								.appendTo(image);
							
							media = $('<audio>', {
									preload: 'auto',
									controls: 'true'
								})
								.one('loadedmetadata', function(e) {
									imageReady();
								})
								.attr('src', Album.getSourcePath(item))
								.appendTo(image);
							
						} else {
							
							// Other
							
							$('<a>', {
									href: Album.getSourcePath(item),
									target: '_blank'
								})
								.append($('<img>', {
									'class': 'other'
								})
								.one('load', function(e) {
									imageReady();
								})
								.attr('src', Album.getImagePath(item)))
								.appendTo(image);
							
							$('<p>', {
									'class': 'click-hint',
									'html': text.clickToOpen
								})
								.appendTo(image);
						}	
					}
					
					prepareCaption(item);
															
				}, options.speed / 4);
			},
			
			// Hiding the lightbox overlay
			
			quitLightbox = function() {
				stopAuto();
				$('body').css('overflow', '');
				
				if (!image.length) {
					if (options.onEnd !== false) {
						options.onEnd(thumb);
					}
				} else {
					lightbox.fadeOut(options.speed, function() {
						if (image.length) {
							image.remove();
							image = $();
						}
						if (caption.length) {
							caption.remove();
							caption = $();
						}
						//inProgress = false;
						if (options.onEnd !== false) {
							options.onEnd(thumb);
						}
					});
				}
			},
			
			// Removing the lightbox completely
			
			removeLightbox = function() {
				stopAuto();
				// removing event handlers
				thumbs.off('.' + ns);
				$(document).add($(window)).add(lightbox).off('.' + ns);
				$('body').css('overflow', '');
				
				if (!image.length) {
					// No image
					lightbox.remove();
				} else {
					// Fade out
					image.animate({ 
						'opacity': 0 
					}, options.speed, function() {
						lightbox.remove();
					});
				}
			},
			
			// Starting auto slideshow
			
			startAuto = function() {
				clearTimeout(slideshow);
				lb_play.hide();
				lb_pause.show();
				hideControlsLater();
				//lb_btn_left.add(lb_btn_right).add(lb_close).add(lb_zoom).fadeOut(1000);
				
				caption.addClass('slideshow');
				slideshow = setTimeout(nextImage, options.slideshowDelay / 4);
				
				if (options.onSlideshowStart !== false) {
					options.onSlideshowStart(thumb);
				}
			},
			
			resumeAuto = function() {
				slideshow = setTimeout(nextImage, options.slideshowDelay / 4);
				lb_pause.find('.progress').show();
			},
	
			stopAuto = function() {
				if (options.onSlideshowPause !== false) {
					options.onSlideshowPause(thumb);
				}
				
				slideshow = clearTimeout(slideshow);
				caption.show().removeClass('slideshow');
				
				lb_play.show();
				lb_pause.hide();
				lb_close.show();
				lb_btn_left.add(lb_btn_right).add(lb_zoom).show();
				
			},
			
			suspendAuto = function() {
				slideshow = clearTimeout(slideshow);
				lb_pause.find('.progress').hide();
				hideControlsLater();
			},
			
			toggleZoom = function() {
				options.fitImages = !options.fitImages;
				fitImage();
			},
			
			showCaption = function() {
				caption.show();
			},
			
			hideCaption = function() {
				caption.fadeOut(1000);
			},
			
			showControls = function() {
				lb_close.add(lb_zoom).show();
			},
			
			hideControls = function() {
				lb_close.add(lb_zoom).fadeOut(1000);
			},
			
			hideControlsLater = function() {
				setTimeout(function() {
						hideControls();
						hideCaption();
					}, slideshow? Math.min(options.slideshowDelay / 2, 2000) : 2000);
			},
			
			showArrows = function() {
				lb_btn_left.add(lb_btn_right).show();
			},
			
			hideArrows = function() {
				lb_btn_left.add(lb_btn_right).fadeOut(1000);
			},
			
			previousImage = function() {
				if (curr > 0) {
					loadImage(curr - 1);
				} else {
					if (options.onBeforeFirst !== false) {
						options.onBeforeFirst();
					}
					quitLightbox();
				}
			},
			
			nextImage = function() {
				if (curr < thumbs.length - 1) {
					loadImage(curr + 1);
				} else {
					if (options.onAfterLast !== false) {
						options.onAfterLast();
					}
					quitLightbox();
				}
			};
			
		// Initializing Lightbox
					
		if (ns) {
			// should never happen, but still living lightbox attached to thumb elements
			cleanup(ns);
		}
		
		// Creating new namespace
		$(this).data('llb_ns', ns = 'llb_' + Math.floor(Math.random() * 10000));		

		// Finding thumbs
		thumbs = $(this).find(options.delegate);
		
		// Setting up the structure
				
		var lightbox = $('<div>', {
					id: 		ns,
					'class': 	'lightbox'
				}).hide().appendTo('body'),
			
			// Darken background
			lb_overlay = $('<div>', {
					'class': 	'lightbox-overlay'
				}).appendTo(lightbox),
			
			// Activity indicator
			lb_activity = $('<div>', {
					'class': 	'lightbox-loading'
				}).append('<div>').appendTo(lightbox),
			
			// Close button
			lb_close = $('<button>', {
					type: 		'button',
					'class': 	'lightbox-btn close',
					'title': 	text.close
				})
				.on('click.' + ns, function(){ 
					quitLightbox(); 
					return false; 
				})
				.appendTo(lightbox),
			
			// Zoom toggle button
			lb_zoom = $('<button>', {
					type: 		'button',
					'class': 	'lightbox-icon zoom icon-expand',
					'title': 	text.toggleZoom
				})
				.on('click.' + ns, function(){
					toggleZoom(); 
					return false; 
				})
				.appendTo(lightbox),
			
			// Play button
			lb_play = $('<button>', {
					type: 		'button',
					'class': 	'lightbox-icon play icon-play',
					'title': 	text.slideshow
				})
				.on('click.' + ns, function(){ 
					startAuto(); 
					return false; 
				})
				.appendTo(lightbox),

			// Pause button
			lb_pause = $('<button>', {
					type: 		'button',
					'class': 	'lightbox-icon pause icon-pause',
					'title': 	text.pause
				})
				.append($('<span>', {
					'class': 	'progress'
				}))
				.hide()
				.on('click.' + ns, function(){ 
					stopAuto(); 
					return false; 
				})
				.appendTo(lightbox),
			
			// Buttons
			lb_btn_left = thumbs.length?
				$('<button>', {
						type: 		'button',
						'class': 	'lightbox-btn arrow-left'
					})
					.on('click.' + ns, function(){
						previousImage();
						return false;
					})
					.appendTo(lightbox)
				: 
				$(),
				
			lb_btn_right = thumbs.length?
				$('<button>', {
						type: 		'button',
						'class': 	'lightbox-btn arrow-right'
					})
					.on('click.' + ns, function(){
						nextImage();
						return false;
					})
					.appendTo(lightbox)
				: 
				$();

		// Setting up events
				
		// Resize
		var resizeTimeout = null;
		
		lightboxWidth = lightbox.width();
		lightboxHeight = lightbox.height();
		
		$(window).on('resize.' + ns /* + ' orientationchange.' + ns*/, function() {
			clearTimeout(resizeTimeout);
			// Executed only if 50ms has passed since the last resize event
			resizeTimeout = setTimeout(function() { 
				clearTimeout(resizeTimeout);
				lightboxWidth = Math.min(lightbox.width(), $(window).width());
				lightboxHeight = Math.min(lightbox.height(), $(window).height());
				//log(lightboxWidth + ':' + lightboxHeight);
				if (image && image.length) {
					fitImage();
				}
			}, 50);
		});
		
		// Quit on document touch
		if (options.quitOnDocClick) {
			lb_overlay.on('click.' + ns, function( e ) {
				if (image.length && !$(e.target).is(image)) {
					quitLightbox();
				}
			});
		}
		
		// Keyboard
		if (options.enableKeyboard) {
			$(document).on('keyup.' + ns, function(e) {
				if (!image.length || lightbox.children('.modal:visible').length) {
					return true;
				}
				
				e.preventDefault();
				
				switch (e.keyCode) {
					
					case 27:
						quitLightbox();
						break;
					
					case 37:
						previousImage();
						break;
						
					case 39:
						nextImage();
						break;
						
					case 107:
						toggleZoom();
						break;
						
					case 32:
						if (isVideo || isAudio) {
							if (media[0].paused) {
								media[0].play();
							} else {
								media[0].pause();
							}
						} else {
							if (slideshow) {
								stopAuto();
							} else {
								startAuto();
							}
						}
						break;
						
					default:
						return true;
										
				}
				
				return false;
			});
		}
		
		// Mouse wheel
		if (options.enableMouseWheel) {
			var wheelTimeout = null;
			$(window).on('mousewheel.' + ns + ' DOMMouseScroll.' + ns, function(event) {
				wheelTimeout = clearTimeout(wheelTimeout);
				if (!image.length || lightbox.children('.modal:visible').length) {
					return true;
				}
				wheelTimeout = setTimeout((event.originalEvent.wheelDelta > 0 || event.originalEvent.detail < 0)? previousImage : nextImage, 100); 
				return false;
			});
		}
		
		// Events that can be triggered from outside
		
			// External trigger to switch image
		$(this).on('lightboxLoad.' + ns, function(e, n) {
				stopAuto();
				if (options.onStart !== false) {
					options.onStart((typeof n === 'number')? thumbs.eq(n) : th);
				}
				loadImage(n);
			})
			// External trigger to quit = does not remove, just hides the lightbox
			.on('lightboxQuit.' + ns, function() {
				quitLightbox();
				return false;
			})
			// Removing the lightbox completely
			.on('lightboxRemove.' + ns, function() {
				removeLightbox();
				return false;
			})
			// Returning the current lightbox container
			.on('lightboxContainer.' + ns, function() {
				return lightbox;
			})
			// External trigger to go to the previous image
			.on('lightboxNext.' + ns, function() {
				stopAuto();
				nextImage();
				return false;
			})
			// External trigger to go to next image
			.on('lightboxPrevious.' + ns, function() {
				stopAuto();
				previousImage();
				return false;
			});
		
		// Adding tooltips
		lightbox.find('button').addTooltip();
		
		// Start
		if (options.hasOwnProperty('auto') && $.isNumeric(options.auto)) {
			if (options.auto < 0) {
				options.auto = thumbs.length + options.auto;
			}
			if (options.onStart !== false) {
				options.onStart(thumbs.eq(options.auto));
			}
			loadImage(options.auto);
		}
		
		// Ready event
		
		if (options.hasOwnProperty('onReady')) {
			options.onReady(thumb);
		}

		return this;
	};
	
	$.fn.lightbox.defaults = {
		delegate:				'li',
		buttonClass:			'secondary button small',
		auto:					false,
		slideshowDelay:			4000,
		fitImages:				false,
		scaleUp:				true,
		fitRatio:				0.94,
		fitAboveCaption:		true,
		mapHeight:				0.8,
		videoAuto:				false,
		volume:					0.5,
		rightClickProtect: 		false,
		allowDownloadOthers:	true,
		printImage:				false,
		speed: 					400,
		preloadNext:			true,
		enableKeyboard: 		true,
		enableMouseWheel:		true,
		quitOnEnd:				true,
		quitOnDocClick: 		true,
		onStart:				false,
		onEnd:					false,
		onLoadStart:			false,
		onLoadEnd:				false,
		onAfterLast:			false,
		onBeforeFirst:			false,
		onReady:				false,
		onSlideshowStart:		false,
		onSlideshowPause:		false
	};
	
	$.fn.lightbox.text = {
		close:					'Close',
		slideshow:				'Slideshow',
		pause:					'Pause',
		toggleZoom:				'Toggle zoom',
		download: 				'Download',
		print:					'Print',
		map: 					'Map',
		buyImage: 				'Buy image',
		photoData: 				'Photo data',
		addCart:				'Add to cart',
		added:					'Added...',
		shoppingcartInfo:		'This item is placed into the "Shopping cart". You can buy it by clicking the "Buy now" button after you quit the lightbox.', 
		clickToOpen:			'Click to open this document with the associated viewer!'
	};

	
})(jQuery);
