;(function($, $window, $document, $body, undefined) {
	'use strict';
				
	/* 
		main.js - the skin's javascript functions
	*/
	
	// Implementing select functions on thumbnails
	
	$.fn.selectable = function(options) {
		
		options = $.extend({
				check: 	'checkbox'
			}, options);
		
		var self = $(this).eq(0),
			thumbs,
		
			getThumbs = function() {
				return options.selector? self.find(options.selector) : self.children();
			},
		
			clicked = function(e) {
				
				var o, li = $(this).closest('li');
				
				if (li.length) {
					li.toggleClass(J.ACTIVE);
				}
				
				return false;
			},
		
			selectAll = function() {
				if (self.is(':visible')) {
					var o;
					thumbs.not('.' + J.ACTIVE).each(function() {
						$(this).addClass(J.ACTIVE);
					});
					return false;
				}
			},
		
			selectNone = function() {
				if (self.is(':visible')) {
					var o;
					thumbs.filter('.' + J.ACTIVE).each(function() {
						$(this).removeClass(J.ACTIVE);
					});
					return false;
				}
			},
			
			selectInverse = function() {
				if (self.is(':visible')) {
					var o, s;
					thumbs.each(function() {
						$(this).toggleClass(J.ACTIVE);
					});
					return false;
				}
			},
			
			refresh = function() {
				thumbs = getThumbs();
				thumbs.each(function() {
						
					if (!($(this).children('span.' + options.check).length)) {
				
						$(this).append($('<span>', {
								'class': 	options.check
							}).on({
								click: 		clicked
							}));
					}
					
				});
			};
		
		refresh();
		
		self.on({
			selectAll: 		selectAll,
			selectNone: 	selectNone,
			selectInverse: 	selectInverse,
			refresh: 		refresh
		});
				
		return this;
	};
			
	// The main skin code
	
	$.fn.skin = function(settings) {
		
		settings = $.extend({}, $.fn.skin.defaults, settings);
		
		var content = 		$(settings.contentHook),				// Main content
			text = 			getTranslations({						// Translated texts
								foundNTimes:			'found {0} time(s)',
								notFound: 				'not found',
								search: 				'Search',
								newImages:				'New images',
								label: 					'Label',
								addCart:				'Add to cart',
								'return': 				'return',
								inTheLastDay:			'in the last day',
								inThePastNDays:			'in the past {0} days',
								sinceMyLastVisit:		'since my last visit',
								imagesAdded:			'Images added',
								imagesModified:			'Images modified',
								imagesTaken:			'Images taken',
								cookiePolicyText: 		'This album uses cookies to remember user preferences. By using it, you agree to our use of cookies.',
								cookiePolicyAgree: 		'Got it',
								cookiePolicyLearnMore: 	'Learn more',
								more:					'more',
								less:					'less'
							}),
			
			shopRoot,												// Root element for cart
			lightbox,												// Lightbox element
			ns = 			'pb_skin',								// Namespace
			lastHash = 		'',										// Storing last state
			initByHash = 	window.location.hash !== "",			// Initial URL contains hash

			// Managing overlays
			
			// Adding new overlay
		
			addOverlay = function(opt) {
			
				content.children('article').each(function() {				
					if (this.hasOwnProperty('overlay')) {
						var ns = $(this).data('oly_ns');
						if (ns) {
							$window.off('.' + ns);
						}
						$(this).find('.' + settings.thumbsClass).trigger('lightboxRemove');
						$(this).remove();
					} else {
						$(this).hide();
					}
				});
				
				var ns = 'ths_' + Math.floor(Math.random() * 10000),
					t = $('<article>', {
							'class': 	(opt.hasOwnProperty('articleClass')? (opt.articleClass + ' ') : '') + 'cont overlay',
							id: 		ns
						}).data('oly_ns', ns).appendTo(content);
				
				t[0]['overlay'] = true;
				
				t = $('<section>', {
						'class': 	settings.thumbContClass
					}).appendTo(t);
				
				var h = $('<header>', {
						'class': 	(opt.hasOwnProperty('icon'))? opt.icon : ''
					}).appendTo(t);
				
				if (opt.hasOwnProperty('title')) {
					h.append($('<h4>', {
							html: 	opt.title
						}));
				}			
						
				h.append($('<a>', {
						'class': 	'close',
						href: 		'',
						text: 		text.return
					}).on('click', function() {
						removeOverlay();
						return false;
					}));
					
				return t;
			},
		
			// Removing overlay
			
			removeOverlay = function() {
				
				content.children('article').each(function() {
					if (this.hasOwnProperty('overlay')) {
						var ns = $(this).data('oly_ns');
						if (ns) {
							$window.off('.' + ns);
						}
						$(this).find('.' + settings.thumbsClass).trigger('lightboxRemove');
						$(this).remove();
					} else {
						$(this).show();
						equalHeight($('.' + settings.foldersClass + '>li'));
					}
				});
				
				removeParam(settings.indexName);
				updateShares();
				
				return null;
			},
			
			// Getting active overlay
			
			getOverlay = function() {
				
				var a = content.children('article:visible').eq(0),
					t = a.children('section.' + settings.thumbContClass);
					
				if (!t.length) {
					t = $('<section>', {
							'class': 	settings.thumbContClass
						}).appendTo(a);
				}
				
				if (!a.data('oly_ns')) {
					var ns = 'oly_' +  + Math.floor(Math.random() * 10000);
					a.data('oly_ns', ns).attr('id', ns);
				}
				
				return t;
			},
		
			// Getting current thumbs array
			currentThumbs = function() {
				
				return content.find('article:visible section.' + settings.thumbContClass + ' .' + settings.thumbsClass);
			},
			
			getThumbByName = function(name) {
				var t;
				content.find('article:visible section.' + settings.thumbContClass + ' .' + settings.thumbsClass + '>li>a:first-child').each(function() {
					if (($(this).attr('href') || '').replace('slides/', '') === name) {
						t = $(this);
						return false;
					}
				});
				return t? t.parent() : $();
			},
			
			// Getting selected thumbs as items
			
			getSelectedItems = function(clear) {
				
				var t = content.find('article:visible section.' + settings.thumbContClass),
					r = [];
						
				if (!t || !(t = t.find('.' + settings.thumbsClass))) {
					return r;
				}
				
				t.children('.' + J.ACTIVE).each(function() {
					if (clear) {
						$(this).removeClass(J.ACTIVE);
					}
					r.push($(this).data(J.OBJ));
				});
				
				return r;
			},
			
			// Getting all items
			
			getAllItems = function() {
				
				var t = content.find('article:visible section.' + settings.thumbContClass),
					r = [];
						
				if (!t || !(t = t.find('.' + settings.thumbsClass))) {
					return r;
				}
				
				t.children().each(function() {
					r.push($(this).data(J.OBJ));
				});
				
				return r;
			},
			
			// Making preloaded folder thumbs equal height
			
			equalHeight = function(thumbs) {
				if (thumbs.length < 2) {
					return;
				}
				
				var lastY = 0,
					row = $(),
					
					makeEqual = function(row) {
						var h, 
							maxHeight = 0;
						
						row.each(function() {
							h = $(this).height();
							if (h > maxHeight) {
								maxHeight = h;
							}
						});
						
						row.height(maxHeight);
					};
				
				thumbs.css('height', 'auto');
				
				thumbs.each(function() {
					var self = $(this),
						top = Math.floor(self.offset().top);
					//console.log(row.length + ':' + top);
					if (row.length) {
						// already elements in a row
						if (top > lastY) {
							if (row.length === settings.folderCols) {
								makeEqual(row);
							}
							lastY = top;
							row = self;
						} else {
							row = row.add(self);
						}
					} else {
						// empty
						lastY = top;
						row = self;
					}
				});
				
				// final row
				if (row.length > 1) {
					makeEqual(row);
				}
			},
			
			// Matching height of newly loaded folder thumbs
			
			matchHeight = function(thumb) {
				
				var mh = 0,
					row = thumb,
					to = Math.floor(thumb.offset().top),
					el = thumb;
					
				while ((el = el.prev()).length && !el.hasClass('lazyload') && Math.floor(el.offset().top) === to) {
					row = row.add(el);
				}
				
				el = thumb;
				
				while ((el = el.next()).length && !el.hasClass('lazyload') && Math.floor(el.offset().top) === to) {
					row = row.add(el);
				}
				
				if (row.length > 1) {
				
					row.each(function() {
						if ($(this).height() > mh) {
							mh = $(this).height();
						}
					});
					
					row.height(mh);
				}
			},
			
			// Adding one folder's thumb
			
			loadFolderThumb = function(thumb) {
					
				thumb.append($('<img>', {
						alt: 		thumb.data('alt'),
						'class': 	'hide-image'
					})
					// Onload action
					.one('load', function() {
						$(this).addClass('show-image').removeClass('hide-image');
						matchHeight(thumb.closest('li').height('auto'));
					})
					.attr('src', thumb.data('thumb')));
									
			},
			
			// Loading visible folder thumbs
			
			lazyloadFolders = function() {
				
				var folders = $('.' + settings.foldersClass).children('.lazyload');
				
				if (folders) {
					var wh = $window.height(),
						scrollTop = $window.scrollTop();
				
					folders.each(function() {
						var tt = $(this).offset().top,
							th = $(this).height();
							
						if ((tt < scrollTop + wh) && (tt + th) > scrollTop) {
							$(this).removeClass('lazyload');
							loadFolderThumb($(this).children('a[data-thumb]').eq(0));
						}
						if (tt > scrollTop + wh) {
							return false;
						}
					});
				}
			};
				

		
		// Rendering thumbnails in a progressive fashion
		
		$.fn.renderThumbs = function(items, complete) {
			
			if (typeof items === UNDEF || !items.length || !$.isArray(items)) {
				return this;
			}
			
			var self = $(this).eq(0),
				cleanup = function(ns) {	
					// cleaning up the old event handlers
					$window.off('.' + ns);
				},
				// Namespace
				ns = $(this).closest('article').data('oly_ns'),
			
				// Scroll timeout
				scrollTimeout = null,
				fixTimeout = null,
				
				// Finding or creating the <ul> element to hold the thumbs
				target = (function() {
					var t = self.find('.' + settings.thumbsClass);
					if (!t.length) {
						t = $('<ul>', {
								'class': 	settings.thumbsClass + ' clearfix'
							}).appendTo(self);
					}
					return t;
				}()),
				
				// Rendering thumbs
				
				initThumbs = function() {
					var thumbs = target.children('li'), 
						thumb,
						src,
						a,
						caption,
						s,
						tw,
						st = new Date(),
						item;
						
					// Creating new elements for the items with no thumbs
					for (var i = 0; i < items.length; i++) {
						item = items[i];
						if (i < thumbs.length) {
							// Already exists in the HTML source
							thumb = thumbs.eq(i);
							a = thumb.find('a.thumb');
						} else {
							// Not yet exists
							thumb = $('<li>').appendTo(target);
							src = Album.getThumbPath(item);
							s = item[J.THUMBCAPTION];
							a = $('<a>', {
									'class': 	'thumb lazyload',
									href: 		Album.getItemPath(item)
								}).appendTo(thumb);
							
							if (s) {
								thumb.addClass('hascaption');
								caption = $('<div>', {
										'class': 	'caption',
										html: 		s
									});
								if (settings.captionAbove) {
									thumb.prepend(caption);
								} else {
									thumb.append(caption);
								}
							}
							
						}
						
						thumb.data(J.OBJ, item);
						
						if (settings.fixGrid) {
							a.height((tw || (tw = thumb.width())) * item[J.THUMB][J.HEIGHT] / item[J.THUMB][J.WIDTH]);
						}
						
						thumb.on('click.' + ns, thumbClicked);
						
						if (shopRoot) {
							a = $('<a>', {
									'class': 	'icon-shopping-cart add-cart',
									text: 		' ' + text.addCart
								}).appendTo(thumb);
								
							a.on('click', function(e) {
									var th = $(e.target).closest('li');
									if (th.length) {
										shopRoot.trigger('addItems', th.data(J.OBJ));
									}
									return false;
								});	
						}
						
						if (settings.rightClickProtect) {
							thumb.on('contextmenu', function(e) {
									e.preventDefault()
									return false;
								});
						}
					}
					
					//console.log('initThumbs() ' + ((new Date()) - st) + 'ms');
				},
				
				cols = 0,
				
				getCols = function() {
					var el = $('<li>', {
								visibility: 	'hidden',
								height: 		0
							}).appendTo(target),
						cols = Math.floor(0.05 + target.width() / el.outerWidth(true)); 
						
					el.remove();
					return cols;
				},
				
				makeAbsolutePositioned = function(thumbs) {
					var th, 
						//st = new Date(),
						pos;
						
					// Already absolute?
					if (thumbs.eq(0).css('position') === 'absolute') {
						return;
					}
					
					// Fixing height
					// target.height(target.height());
					
					// Going backwards
					for (var i = thumbs.length - 1; i >= 0; i--) {
						th = thumbs.eq(i);
						pos = th.position();
						th.css({
								position: 	'absolute',
								left: 		pos.left,
								top: 		pos.top
							});
					}
					
					//console.log('makeAbsolutePositioned() ' + ((new Date()) - st) + 'ms');
				},
				
				// Fixing grid
				fixThumbs = function(init) {
					var nc = getCols(),
						th, 
						thumbs = target.children('li'),
						top = [],
						left = [],
						bottom = 0,
						i,
						cw = thumbs.eq(0).outerWidth(true),
						tw = thumbs.eq(0).width(),
						data,
						belowPos = $body.scrollTop() + $window.height(),
						//st = new Date(),
						getTopCol = function() {
							var l = 0,
								min = top[0];
								
							for (i = 1; i < top.length; i++) {
								if (top[i] < min) {
									min = top[i];
									l = i;
								}
							}
							return l;
						};
					
					cols = nc;
					
					//makeAbsolutePositioned();
					makeAbsolutePositioned(thumbs.slice(0, 3)); 
					thumbs.slice(3).hide();
					
					for (var i = 0; i < cols; i++) {
						top[i] = 0;
						left[i] = i * cw;
					}
					
					thumbs.each(function() {
						i = getTopCol();
						data = $(this).data(J.OBJ);
						
						$(this).css({
								position:	'absolute',
								left: 		left[i],
								top: 		top[i]
							}).show();
						
						if (data) {
							$(this).children('a.thumb').height(tw * data[J.THUMB][J.HEIGHT] / data[J.THUMB][J.WIDTH]);
						}
						
						top[i] += $(this).outerHeight(true);
						bottom = Math.max(bottom, top[i]);
						
						if (belowPos && top[i] > belowPos) {
							lazyloadThumbs();
							belowPos = 0;
						}
					});
					
					target.height(bottom);
					
					if (belowPos) {
						lazyloadThumbs();
					}
					
					//console.log('fixThumbs() ' + ((new Date()) - st) + 'ms');
						
				},
				
				// Loading visible thumbs
				lazyloadThumbs = function() {
					if (!target.is(':visible')) {
						return;
					}
					var thumbs = target.find('.lazyload'),
						wh = $window.height(),
						scrollTop = $window.scrollTop();
					
					thumbs.each(function() {
						var tt = $(this).offset().top,
							th = $(this).height();
						if ((tt < scrollTop + wh) && (tt + th) > scrollTop) {
							loadThumb($(this).removeClass('lazyload'));
						}
						if (tt > scrollTop + wh) {
							return false;
						}
					});
				},
				
				// Clicked thumb
				thumbClicked = function(e) {
					if (e.target.nodeName === 'A') {
						// Link in the caption
						return true;
					}
					target.trigger('lightboxLoad', target.children('li').index($(e.currentTarget)));
					return false;
				},
				
				// Adding one thumb
				loadThumb = function(thumb) {
						
					thumb.append($('<img>', {
							src: 		Album.getThumbPath(thumb.closest('li').data(J.OBJ)),
							'class': 	'hide-image'
						})
						// Onload action
						.one('load', function() {
							$(this).addClass('show-image').removeClass('hide-image');
						}));
										
				},
				
				// Marking active thumb
				setActiveThumb = function(e, thumb) {
					if (!thumb) {
						return;
					}
					
					var thumb = $(thumb),
						wh = 			$window.height(),
						scrollTop = 	$window.scrollTop(),
						tt = 			thumb.offset().top,
						th = 			thumb.outerHeight();
						
					if (tt + th > scrollTop + wh) {
						$('html, body').stop(true, false).animate({
								scrollTop: 		Math.round(tt - wh + th + 30) + 'px'
							});
					} else if (tt < scrollTop) {
						$('html, body').stop(true, false).animate({
								scrollTop: 		Math.round(Math.max(tt - 30, 0)) + 'px'
							});
					}
				};
				
			
			// Creating new Unique ID if not yet exists
			if (!ns) {
				$(this).closest('article').data('oly_ns', ns = 'oly_' + Math.floor(Math.random() * 10000));
			}
				
			self.append($('<div>', {
					'class': 	'progressbar'
				}));
				
			initThumbs();
			
			if (settings.fixGrid) {
				
				fixThumbs(true);
				
				$window.on('resize.' + ns, function(e) {
						clearTimeout(fixTimeout);
						fixTimeout = setTimeout(fixThumbs, 500);
						return true;
					});
			}
			
			lazyloadThumbs();

			$window.on('scroll.' + ns + ' resize.' + ns, function(e) {
				// Avoid multiple calls in short time
				clearTimeout(scrollTimeout);
				scrollTimeout = setTimeout(lazyloadThumbs, 25);
				return true;
			});
			
			// Moving the active element into view
			self.on('setactivethumb', setActiveThumb);
			
			if ($.isFunction(complete)) {
				if (typeof auto !== 'undefined') {
					complete.call(undefined, target, auto);
				} else {
					complete.call(undefined, target);
				}
			}
			
			self.find('.progressbar').remove();
			
			return this;
		};
		
		// Do a label filtering
		
		var getItemsByLabel = function(tx, ready) {
				var t = addOverlay({
							articleClass:	'tags-found',
							icon:			'icon-label',
							title:			'&ldquo;' + tx + '&rdquo;'
						}),
					h = t.find('header h4');
					
				h.append($('<span>', {
					'class': 'progressbar'
				}));
				
				if (t.offset().top < $window.scrollTop() || t.offset().top > ($window.scrollTop() + $window.height() - 100)) {
					$window.scrollTop(t.offset().top);
				}
				
				Album.collectItems($.extend(settings.tagCloud, {
					exact:	true,
					terms:	'"' + tx + '"',
					ready: 	function() {
								h.find('.progressbar').remove();
								h.append($('<small>', {
									text: this.length? text.foundNTimes.replace('{0}', this.length) : text.notFound
								}));
								t.renderThumbs(this, prepareThumbs);
								if (ready) {
									ready.call();
								}
							}
				}));
			},

			// Collecting and displaying tags
		
			renderTagCloud = function(tags) {
				
				if (!tags.length) {
					return;
				}
				
				var t = $(settings.tagCloud.hook);
				
				if (!t.length) {
					return;
				}
				
				var tagClicked = function() {
					var tx = $(this).children('span').eq(0).text();
					
					if (tx && tx.length > 1) {
						getItemsByLabel(tx, function() {
							setParam(settings.indexName, {
									label: 	tx
								}, text.label + ': ' + tx);
							updateShares();
						});
					}
					
					return false;
				};
								
				for (var a, r, cnt, i = 0, l = tags.length; i < l; i++) {
					
					// create link element
					a = $('<a>', {
							'class': 	'tag'
						}).append($('<span>', {
							text: 		tags[i][0]
						}));
					a.on('click', tagClicked);
					
					// add count
					if (tags[i][1] > 1) {
						if (settings.tagCloud.fontVaries) {
							r = 0;
							cnt = Math.min(64, tags[i][1]);
							// calculate weight
							while (cnt > 1) {
								r++;
								cnt = cnt >> 1;
							}
							a.addClass('size-' + r);
						}
						a.append('&nbsp;').append($('<sup>', {
							text: tags[i][1]
						}));
					}
					
					t.append(a);
				}
				
				if (tags.length) {
					var d;
					
					if (t.outerHeight() > 175) {
						
						t.css({
							maxHeight: 150,
							overflow: 'hidden'
						});
						
						d = $('<div>', {
							'class': 'toggle-height'
						});
						
						d.append($('<a>', {
								'class': 	'icon-drop-down',
								text: 		' ' + text.more
							}).on('click', function() {
								var b = t.parent(),
									toggleHeight = function() {
										var h = t.prop('scrollHeight');
										
										b.css({
												height: 	'auto',
												overflow: 	'visible'
											});
										t.css({
												maxHeight: 	150,
											});
										setTimeout(function() {
												t.css({
														maxHeight: 	h + 20
													});
											}, 25);
										d.addClass('active');
									};
								
								b.css({
										height: 	b.height(),
										overflow: 	'hidden'
									});
								t.css({
										maxHeight: 	'none'
									});
								setTimeout(toggleHeight, 50);
							
						})).append($('<a>', {
								'class': 	'icon-drop-up',
								text: 		' ' + text.less
							}).on('click', function() {
								t.css({
										maxHeight: 	150
									});
								d.removeClass('active');
							}));
						
						t.after(d);
					}
				}
			},
			
			getFolderTree = function() {
				var getFolders = function(folder) {
						if (folder.hasOwnProperty(J.FOLDERS) && folder[J.FOLDERS].length) {
							var ul, li, i, p, f = folder[J.FOLDERS];
							ul = $('<ul>');
							for (i = 0; i < f.length; i++) {
								p = Album.getRelativeFolderPath(f[i]);
								li = $('<li>').appendTo(ul);
								li.append($('<a>', {
										href: 	(p? (p + '/') : '') + settings.indexName,
										html: 	f[i][J.TITLE] || f[i][J.NAME] || ''
									})).append(getFolders(f[i]));
								if (!p) {
									li.addClass('actual');
								}
							}
							return ul;
						}
						return $();
					},
					getHomeLink = function() {
						var p = settings.rootPath;
						return $('<a>', {
								'class': 	'home-link icon-home',
								href:		(p? (p + '/') : '') + settings.indexName,
								text:		' ' + Album.getAlbumTitle()
							});
					};
					
				if (settings.folderTree.type === 'tree') {
					if (settings.level > 0) {
						return getHomeLink().add(getFolders(Album.getTree()));
					} else {
						return getFolders(Album.getTree());
					}
				} else {
					getFolders(Album.getCurrentFolder());
				}
			},
		
			// Executing Search
			
			getItemsBySearch = function(tx, ready) {
				var t = addOverlay({
							articleClass:	'search-results',
							icon:			'icon-search',
							title:			'&ldquo;' + tx + '&rdquo;'
						}),
					h = t.find('header h4');
					
				h.append($('<span>', {
						'class': 	'progressbar'
					}));
				
				setTimeout(function() {
					Album.collectItems($.extend(settings.search, {
						terms:		tx,
						ready:		function() {
										h.find('.progressbar').remove();
										h.append($('<small>', {
												text: 	this.length? text.foundNTimes.replace('{0}', this.length) : text.notFound
											}));
										t.renderThumbs(this, prepareThumbs);
										if (ready) {
											ready.call();
										}
									}
					}));
				}, 50);
			},
	
		
			// Preparing Search functionality
			
			prepareSearch = function(f) {
				
				if (!f.length) {
					return;
				}
				
				// Adding defaults if missing
				
				settings.search = $.extend({
					depth:		'tree',
					fields:		'creator,keywords,title,comment'
				}, settings.search);
				
				var ACT = 'active',
					
					startSearch = function(f) {
						
						var tx = f.find('input').eq(0).val();
						
						if (tx && tx.length > 1) {
							getItemsBySearch(tx, function() {
								setParam(settings.indexName, {
										search:		tx
									}, text.search + ': ' + tx);
								updateShares();
							});
						}
						return false;
					};
					
				f.find('.search-btn, button').on('click', function() {
					if (f.hasClass(ACT)) {
						startSearch(f);
						f.removeClass(ACT);
					} else {
						f.find('input').eq(0).focus();
						f.addClass(ACT);
					}
					return false;
				});
				
				f.on('submit', function(e) {
					e.preventDefault();
					startSearch(f);
					f.removeClass(ACT);
					return false;
				});
				
				f.find('input').on({
					'focus': function() { 
						f.addClass(ACT); 
					}
				});
			},
			
			// Preparing Search functionality
			
			prepareTagSearch = function() {
				
				var f = $(settings.tagCloud.hook ).siblings('form');
				
				if (!f.length) {
					return;
				}
								
				var startSearch = function(f) {
						
						var tx = f.find('input').eq(0).val();
						
						if (tx && tx.length > 1) {
							getItemsByLabel(tx, function() {
								setParam(settings.indexName, {
										label: 	tx
									}, text.label + ': ' + tx);
								updateShares();
							});
						}
						return false;
					};
					
				f.find('.search-btn, button').on('click', function() {
					f.find('input').eq(0).focus();
					startSearch(f);
					return false;
				});
				
				f.on('submit', function(e) {
					e.preventDefault();
					startSearch(f);
					return false;
				});
				
			},
			
			// Search for new images
			
			getSearchNewTitle = function(r) {
			
				if (r == 'dateTaken') 
					return text.imagesTaken;
				else if (r == 'fileModified')
					return text.imagesModified;
				return text.imagesAdded;
			},
			
			getItemsByDate = function(range, ready) {
				var t = addOverlay({
							articleClass:	'search-results',
							icon:			'icon-star',
							title:			getSearchNewTitle(settings.searchNew.reference) + ' ' + getTimespan(range)
						}),
					h = t.find('header h4');
					
				h.append($('<span>', {
					'class': 'progressbar'
				}));
				
				setTimeout(function() {
					Album.collectByDate($.extend(settings.searchNew, {
						range:	range,
						ready:	function() {
									h.find('.progressbar').remove();
									h.append($('<small>', {
										text: this.length? text.foundNTimes.replace('{0}', this.length) : text.notFound
									}));
									t.renderThumbs(this, prepareThumbs);
									if (ready) {
										ready.call();
									}
								}
					}));
				}, 50);
			},
	
			// Search new images
			
			searchNew = function(range) {
				
				if (range) {
					getItemsByDate(range, function() {
						setParam(settings.indexName, {
								newimages:	range
							}, getSearchNewTitle(settings.searchNew.reference) + ' ' + getTimespan(range));
						updateShares();
					});
				}
				return false;
			},
			
			// Prepare Search new functionality
			
			prepareSearchNew = function(f) {
				
				if (!f.length) {
					return;
				}
				
				// Setting up defaults if something is missing
				
				settings.searchNew = $.extend({
					days: 			'3,30,90',
					depth: 			'folder',
					reference: 		'added',
					sinceLastVisit: true
				}, settings.searchNew);
				
				var	days = settings.searchNew.days.split(',');
				
				f.append($('<span>', {
						text: 	getSearchNewTitle(settings.searchNew.reference)
					}));
						
				for (var i = 0; i < days.length; i++) {
					f.append($('<a>', {
							'class': 	'btn',
							text: 		getTimespan(days[i])
						})
						.data('days', parseInt(days[i], 10))
						.on('click', function() {
							searchNew($(this).data('days'));
						}));
				}
						
				if (settings.searchNew.sinceLastVisit) {
					
					var start = $.cookie('lastVisit'),
						today = Math.ceil((new Date()) / 86400000);
					
					if (start && start < today) {
						f.append($('<a>', {
								'class': 	'btn',
								text: 		text.sinceMyLastVisit
							})
							.on('click', function() {
								searchNew(today - start);
							}));
					}
					
					// Storing last visit date for 10 years
					$.cookie('lastVisit', today, 315360000);
				}
				
			},
				
			// Preparing thumbnails
					
			prepareThumbs = function(thumbs, auto) {
				
				if (settings.hasOwnProperty('shop')) {
					thumbs.selectable();
				}
				
				thumbs.trigger('lightboxRemove');
				thumbs.lightbox({
					auto: 			(typeof auto === 'undefined')? false : auto, 
					onLoadStart:	function(el) {
										var item = el.data(J.OBJ);
										if (item) {
											addParam(settings.indexName, {
												img: Album.getItemName(item)
											}, item[J.TITLE]);
										}						
									},
					onLoadEnd: 		function(el) {
										thumbs.trigger('setactivethumb', el)
									},
					onEnd:			function() {
										removeParam(settings.indexName, 'img');
									},
					onReady:		function() {
										if (initByHash && window.location.hash) {
											initByHash = false;
											var param = readParam();
											
											if (param.hasOwnProperty('label')) {
												// Label filter
												getItemsByLabel(param.label, function() {
													addParam(settings.indexName, {
														label:		param.label,
														search:		null,
														newimages:	null
													}, text.label + ': ' + param.label);
												});
											} else if (param.hasOwnProperty('search')) {
												// Search
												getItemsBySearch(param.search, function() {
													addParam(settings.indexName, {
														search:		param.search,
														label:		null,
														newimages: 	null
													}, text.search + ': ' + param.search);
												});
											} else if (param.hasOwnProperty('newimages')) {
												// Search
												getItemsByDate(param.newimages, function() {
													addParam(settings.indexName, {
														newimages:	param.newimages,
														search:		null,
														label:		null
													}, text.searchNew + ': ' + param.searchNew);
												});
											} else if (param.hasOwnProperty('img')) {
												var el = getThumbByName(param.img);
												if (el.length) {
													currentThumbs().trigger('lightboxLoad', el);
												} else {
													removeParam(settings.indexName, 'img');
												}
											}
										}
									},
					onSlideshowStart:
									function() {
										if (settings.hasOwnProperty('backgroundMusic') && settings.backgroundMusic.slideshowControl) {
											$(settings.backgroundMusic.hook).trigger('startPlayer');
										}
									},
					onSlideshowPause:
									function() {
										if (settings.hasOwnProperty('backgroundMusic') && settings.backgroundMusic.slideshowControl) {
											$(settings.backgroundMusic.hook).trigger('pausePlayer');
										}
									}
				});
			},
			
			// Reading out map locations
			
			getMarkers = function(images) {
				
				var markers = [];
				
				for (var i = 0; i < images.length; i++) {
					if (images[i].hasOwnProperty('location')) {
						markers.push(images[i].hasOwnProperty('title')? {
							title:	images[i].title,
							pos: 	images[i].location,
							link:	images[i].path
						} : {
							pos: 	images[i].location,
							link:	images[i].path
						});
					}
				}
				
				return markers;
							
			},
		
			// Preparing the album upon Album object is loaded
			
			prepareAlbum = function() {
				
				var images = Album.getImages()
				
				if (settings.hasOwnProperty('skipFirst')) {
					images = images.slice(settings.skipFirst);
				}
				
				// Shopping cart
				var s = Album.getRootProperty(J.SHOP);
				
				try {
					s = JSON.parse(s);
				} catch (e) {
					s = null;
				}
				
				if (s) {
					
					settings.shop = s;
					
					// assigning paypal.js defaults 
					$.fn.paypal.defaults.relPath = settings.relPath;
					$.fn.paypal.defaults.rootPath = settings.rootPath;
					
					// Initializing the cart
					shopRoot = $body.paypal(settings.shop);
					
					if (shopRoot) {
						//console.log('shopRoot ready');
						
						// "Select all" button
						$(settings.shop.buttons.selectAll).on('click', function() {
							return currentThumbs().trigger('selectAll');
						});
						
						// "Select none" button
						$(settings.shop.buttons.selectNone).on('click', function() {
							return currentThumbs().trigger('selectNone');
						});
						
						// "Add selected" button
						$(settings.shop.buttons.addSelected).on('click', function() {
							shopRoot.trigger('addItems', getSelectedItems(true));
							currentThumbs().trigger('selectNone');
							return false;
						});
						
						// "Add all" button
						$(settings.shop.buttons.addAll).on('click', function() {
							shopRoot.trigger('addItems', getAllItems());
							return false;
						});
						
						// Ensuring lightbox has 'shop' property
						if (!$.fn.lightbox.defaults.hasOwnProperty('shop')) {
							$.fn.lightbox.defaults.shop = {};
						}
						// Setting lightbox's shop root variable
						$.fn.lightbox.defaults.shop.root = shopRoot;
					}
				}

				// Rendering thumbnails
				if (images.length) {
					getOverlay().renderThumbs(images, prepareThumbs);
					
					// Collecting markers and initializing map
					if (settings.hasOwnProperty('map') && settings.map.index) {
						$('.map-root').addMap({
							markers: 	getMarkers(images),
							type:		settings.map.indexType || 'roadmap',
							click:		function() {
											if (this.link) {
												var el = getThumbByName(this.link);
												if (el.length) {
													removeOverlay();
													currentThumbs().trigger('lightboxLoad', el);
												}
											}
										}
						});
					}

				} else if (window.location.hash) {
					// no thumbnails, only folders
					// we should check the search / tags / newimages filter
					stateChange();
				}
				
				// Collecting tags
				if (settings.hasOwnProperty('tagCloud')) {
					prepareTagSearch(settings.tagCloud);
					Album.collectTags($.extend(settings.tagCloud, {
						ready:	function() {
									renderTagCloud(this);
								}
					}));
				}
				
				// Folder tree
				if (settings.hasOwnProperty('folderTree')) {
					$(settings.folderTree.hook).append(getFolderTree());
				}
				
			},
			
			// Preparing a custom page
			
			preparePage = function() {
				// Placeholder for page initializing functions
			},
			
			// Preparing folders
		
			prepareFolders = function() { 
				
				var scrollTimeout = null;
				
				if (settings.folderCols > 1) {
					$('.' + settings.foldersClass + '>li').each(function(i) {
						//console.log(i);
						if ((i % settings.folderCols) === 0) {
							$(this).css('clear', 'both');
						}
					});
					// Waiting all preloaded thumbs to load and making them equal height
					$('.' + settings.foldersClass + '>li.preload img').waitAllImg(function() {
						setTimeout(function() {
							equalHeight($('.' + settings.foldersClass + '>li.preload'));
						}, 100);
					});
				}
								
				// Lazy loading folder thumbnails for the current view
				lazyloadFolders();
				
				$window.on('scroll resize orientationchange', function(e) {
					// Avoid multiple calls in short time
					clearTimeout(scrollTimeout);
					scrollTimeout = setTimeout(lazyloadFolders, 50);
					return true;
				});
				
				if (settings.folderCols > 1) {
					$window.on('resize orientationchange', function(e) {
						if (Foundation.MediaQuery.atLeast('medium')) {
							// Multi-column
							equalHeight($('.' + settings.foldersClass + '>li'));
						} else {
							// 1 column
							$('.' + settings.foldersClass + '>li').css('height', 'auto');
						}
						return true;
					});
				}
				
			},
			
			// Cookie policy
		
			showCookiePolicy = function() {
			
				if (!$.cookie('cookiePolicy')) {
					var	el = $('<div>', { 
								id: 	'cookiepolicy' 
							}).appendTo($body),
						p = $('<p>', { 
								html: 	text.cookiePolicyText 
							}).appendTo(el);
					
					if (settings.cookiePolicyUrl) {
						p.append($('<a>', { 
								text: 	text.cookiePolicyLearnMore, 
								target: '_blank', 
								href: 	settings.cookiePolicyUrl 
							}));
					}
					
					p.append($('<a>', {
							'class': 	'btn',
							text: 		text.cookiePolicyAgree 
						}).on('click', function(){ 
							$('#cookiepolicy').fadeOut(500, function() { 
								$(this).remove(); 
							});
							$.cookie('cookiePolicy', true, 36000000); // expires: 10000 days
						}));
					
					el.fadeIn(500);
					setTimeout(function() {
						$('#cookiepolicy').fadeOut(500, function() { 
							$(this).remove(); 
						});
					}, 6000);
				}			
			},
			
			// Hash change listener :: when the browser's back / forward buttons were pressed
			
			stateChange = function() {
				if (window.location.hash === lastHash) {
					return;
				}
				
				lastHash = window.location.hash;
				var param = readParam();
				
				if (param.hasOwnProperty('label')) {
					// Label filter
					getItemsByLabel(param.label, function() {
						setParam(settings.indexName, {
								label:	param.label
							}, text.label + ': ' + param.label);
						updateShares();
					});
				} else if (param.hasOwnProperty('search')) {
					// Search
					getItemsBySearch(param.search, function() {
						setParam(settings.indexName, {
								search:	param.search
							}, text.search + ': ' + param.search);
						updateShares();
					});
				} else if (param.hasOwnProperty('newimages')) {
					// New images
					getItemsByDate(param.newimages, function() {
						setParam(settings.indexName, {
								newimages: param.newimages
							}, text.newImages + ': ' + param.newimages);
						updateShares();	
					});
				} else if (param.hasOwnProperty('img')) {
					// Image (no search or label)
					var el = getThumbByName(param.img);
					if (el.length) {
						currentThumbs().trigger('lightboxLoad', el);
					} else {
						removeParam(settings.indexName, 'img');
					}
				} else {
					// Plain index page
					currentThumbs().trigger('lightboxQuit');
					removeOverlay();
				}
				
			},
			
			// Updating the share buttons after a state change
			
			updateShares = function() {
				if (settings.hasOwnProperty('share') && currentThumbs().trigger('lightboxContainer').is(':visible')) {
					$(settings.share.hook).trigger('updateLinks');
				}	
			};
			
		
		// Initializing album
		
		readUserPrefs(settings, 'fitImages');
		
		if (typeof Album !== UNDEF) {
			Album.init({
				indexName:	settings.indexName,
				rootPath:	settings.rootPath,
				relPath:	settings.relPath,
				ready: 		(settings.pageType === 'index')? prepareAlbum : preparePage
			});
		}
		
		if (settings.hasOwnProperty('showCookiePolicy') && settings.showCookiePolicy) { 
			setTimeout(showCookiePolicy, 2000);
		}
		
		if (window.location.search) {
			removeSearch();
		}
		
		if ($('.' + settings.foldersClass).length) {
			prepareFolders();
		}
		
		if (settings.rightClickProtect) {
			$('.folder-cont img, .slider img').on('contextmenu', function(e) {
				e.preventDefault()
				return false;
			});
		}
		
		if (settings.hasOwnProperty('search')) {
			prepareSearch($(settings.search.hook));
		}
		
		if (settings.hasOwnProperty('share')) {
			$(settings.share.hook).renderShares(settings.share);
			$.fn.lightbox.defaults.share = settings.share.sites;
		}
		
		if (settings.hasOwnProperty('searchNew')) {
			prepareSearchNew($(settings.searchNew.hook));
		}
		
		if (settings.hasOwnProperty('backgroundMusic')) {
			$(settings.backgroundMusic.hook).audioPlayer($.extend({
					rootPath: settings.rootPath
				}, settings.backgroundMusic)
			);
		}
					
		// Initializing lightbox defaults
		
		if (settings.hasOwnProperty('map')) {
			if (settings.map.hasOwnProperty('type')) {
				$.fn.lightbox.defaults.mapType = settings.map.type;
				$.fn.addMap.defaults.type = settings.map.type;
			}
			if (settings.map.hasOwnProperty('zoom')) {
				$.fn.lightbox.defaults.mapZoom = settings.map.zoom;
				$.fn.addMap.defaults.zoom = settings.map.zoom;
			}
			if (settings.map.hasOwnProperty('apiKey')) {
				$.fn.addMap.defaults.apiKey = settings.map.apiKey;
			}
		}
		
		// Passing along lightbox defaults
		
		passDefaults(settings, $.fn.lightbox.defaults, 'printImage,speed,slideshowDelay,fitImages,scaleUp,fitAboveCaption,extraSizes,videoAuto,rightClickProtect,enableKeyboard,enableMouseWheel');
		
		// Handled by external libraries
		
		if (typeof Newsfeed !== UNDEF && settings.hasOwnProperty('newsfeed')) {
			Newsfeed.init(settings.newsfeed);
		}

		if (typeof Instafeed !== UNDEF && !LOCAL && settings.hasOwnProperty('instagram')) {
			var feed = new Instafeed({
				get:			'user',
				userId: 		parseInt(settings.instagram.token.split('.')[0], 10),
				accessToken:	settings.instagram.token,
				limit: 			settings.instagram.limit
			});
			feed.run();
		}
				
		$window.on('hashchange', stateChange);
		
	};
	
	$.fn.skin.defaults = {
		articleHook: 		'article#main-cont',
		thumbContClass:		'thumb-cont',
		foldersClass:		'folders',
		folderCols:			2,
		indexName:			'index.html',
		thumbsClass: 		'thumbnails',
		fixGrid:			false,
		fitImages:			false,
		rightClickProtect: 	false,
		enableKeyboard:		true,
		videoAuto:			true
	};
		
})(jQuery, $(window), $(document), $('body'));
